/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     MyServerSessionPool.java                                   *
 *                                                                           *
 *  DESCRIPTION:  A very simple implementation of the JMS ServerSessionPool  *
 *                interface. It is used to pool a number of ServerSessions   *
 *                for use by a ConnectionConsumer. It is static in that a    *
 *                given number of ServerSessions are created during constr-  *
 *                uction of the pool, and the pool is unable to grow or      *
 *                shrink dynamically after construction.                     *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        n/a                                                        *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;

public class MyServerSessionPool implements ServerSessionPool {

   private int               capacity;            // The capacity of this pool
   private MyServerSession[] serverSession;           // Storage for this pool
   private boolean[]         inUse;      // In-Use flag for each ServerSession
   private boolean           quit;                         // 'Stop pool' flag

   private static final int  NONE_AVAILABLE = -1;

   /**************************************************************************
    *  MyServerSessionPool - Constructor. It creates 'capacity' Sessions of  *
    *                        the given acknowledge mode type from the        *
    *                        Connection it is supplied with, and uses these  *
    *                        in creating a static pool of ServerSessions.    *
    *                                                                        *
    *                        Each session has a message listener created     *
    *                        from the given MessageListenerFactory register- *
    *                        ed with it.                                     *
    **************************************************************************/
   public MyServerSessionPool( Connection connection, int capacity,
                               int ackMode, MessageListenerFactory mlf )
      throws JMSException
   {
      // Allocate storage for the pool
      this.capacity = capacity;
      serverSession = new MyServerSession[capacity];

      try {
         if( connection instanceof QueueConnection ) {
            // The given Connection is a point-to-point one, so create
            // QueueSessions for our ServerSessions
            for( int i=0; i<capacity; i++ ) {
               Session s=((QueueConnection)connection).createQueueSession(
                                                            false, ackMode );
               s.setMessageListener(mlf.createMessageListener());
               serverSession[i]=new MyServerSession(this, i, s);
            }
         } else {
            // The given Connection is a publish/subscribe one, so create
            // TopicSessions for our ServerSessions
            for(int i=0; i<capacity; i++) {
               Session s=((TopicConnection)connection).createTopicSession(
                                                            false, ackMode );
               s.setMessageListener(mlf.createMessageListener());
               serverSession[i]=new MyServerSession(this, i, s);
            }
         }

         // Create inUse array, whose elements default to false
         inUse = new boolean[capacity];

      } catch( JMSException e ) {
         // Destroy any successfully-created sessions
         for( int i=0; i<capacity; i++ ) {
            if( serverSession[i]!=null ) serverSession[i].close();
         }
         throw(e);
      }
   }

   /**************************************************************************
    *  getServerSession - Implements javax.jms.ServerSessionPool. It will    *
    *                     attempt to retrieve a ServerSession from the pool. *
    *                     If one isn't available, this method will block     *
    *                     until one is returned to the pool via invocation   *
    *                     of the serverSessionFinished method                *
    **************************************************************************/
   public ServerSession getServerSession() throws JMSException {
      int chosen = NONE_AVAILABLE; // Array index of the chosen ServerSession

      synchronized(this) {
         // All the time we are not being told to quit and have not obtained
         // an unused ServerSession, perform the following loop
         while( !quit && chosen==NONE_AVAILABLE ) {
            for( int i=0; i<capacity && chosen==NONE_AVAILABLE; i++ ) {
               // If we find an unused ServerSession, select it
               if( !inUse[i] ) chosen = i;
            }

            if( chosen==NONE_AVAILABLE ) {
               try {
                  // We haven't been able to find an available ServerSession,
                  // so wait until we are notified either (a) to quit, or
                  // (b) that a ServerSession has returned to the pool
                  wait();
               } catch( InterruptedException ie ) {
                  // Ignore
               }
            }
         }

         // If we have selected an available ServerSession, update its status
         // to 'in use'
         if( chosen!=NONE_AVAILABLE ) inUse[chosen] = true;
      }

      if( chosen!=NONE_AVAILABLE ) {
         // We have a valid index, so return the ServerSession in question
         return( serverSession[chosen] );
      } else {
         // If we've escaped from the main loop of this method, but 'chosen'
         // is not a valid index, then we must be shutting down
         throw new JMSException("MyServerSessionPool is shutting down");
      }
   }

   /**************************************************************************
    *  serverSessionFinished - Callback method used by an individual Server- *
    *                          Session owned by this pool to notify us that  *
    *                          it is eligible for reuse and should be ret-   *
    *                          urned to the pool.                            *
    **************************************************************************/
   void serverSessionFinished( int id ) {
      synchronized(this) {
         inUse[id]=false;
         notifyAll();
      }
   }

   /**************************************************************************
    *  close - Method to signify that this ServerSessionPool should shut     *
    *          down along with all the ServerSessions it is pooling.         *
    **************************************************************************/
   public void close() {
      synchronized(this) {
         quit=true;
         notifyAll();
      }

      // Remember to close down all the ServerSessions in the pool
      for(int i=0; i<capacity; i++) {
         serverSession[i].close();
      }
   }
}
